/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: ui.h
 * Date: 02.12.2013
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Implements user interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/


#ifndef _UI_HEADER_FILE_
#define _UI_HEADER_FILE_

#include <xparameters.h>

/*
 * Uart Lite for MicroBlaze
 */
#if defined __MICROBLAZE__
#   include <xuartlite_l.h>
#   define UART_sendByte(base, data)  XUartLite_SendByte(base, data)
#   define UART_RecvByte(base) XUartLite_RecvByte(base)
#   define UART_GetStatusReg(base) XUartLite_GetStatusReg(base)
#   define UART_SR_RX_FIFO_VALID_DATA  XUL_SR_RX_FIFO_VALID_DATA
#   define STDIN    STDIN_BASEADDRESS
#   define STDOUT   STDOUT_BASEADDRESS
#   define UART_IsReceiveData(base) !(XUartLite_IsReceiveEmpty(base))
#   define UART_RecvByte_nb(base) XUartLite_ReadReg(base, XUL_RX_FIFO_OFFSET);

/*
 * Zynq Uart PS
 */
#else
#   include <xuartps.h>
#   include <xuartps_hw.h>
#   define UART_sendByte(base, data)  XUartPs_SendByte(base, data)
#   define UART_RecvByte(base) XUartPs_RecvByte(base)
#   define UART_GetStatusReg(base) ((Xil_In32(base + XUARTPS_SR_OFFSET) & XUARTPS_SR_RXEMPTY) ? 0:1)
#   define UART_SR_RX_FIFO_VALID_DATA  1
#   define STDIN    STDIN_BASEADDRESS //XPS_UART1_BASEADDR
#   define STDOUT   STDOUT_BASEADDRESS  //XPS_UART1_BASEADDR
#   define UART_IsReceiveData(base) XUartPs_IsReceiveData(base)
#   define UART_RecvByte_nb(base) XUartPs_ReadReg(base, XUARTPS_FIFO_OFFSET);
#endif

#define KEY_ESC   (-27)   /* ascii codes returned by function ui_get_digit */
#define KEY_ENTER (-13)


#define NO_KEY          255

#define KEY_SEL_TIMEOUT 9999999


struct ui_menu_item {
  u8    key;
  char  *item;
};
typedef struct ui_menu_item ui_menu_t;



unsigned char ui_getch(void);
void ui_putch(unsigned char ch);
unsigned char ui_getch_nb(void);
int ui_get_digit(unsigned int base);
int ui_get_digit_nb(unsigned int base);
void ui_put_digit(int dig);
int ui_get_value(const char *text);
int ui_get_address(int bits);
int ui_get_integer(int digs);
int ui_get_float(int decdig, float *retval);

char *ui_conv_to_bin(unsigned long val, int bits);

void ui_print_menu(const ui_menu_t menu[], unsigned char *print_en);

#endif /* _UI_HEADER_FILE_ */
